library flutter_compute_tree;

import 'dart:async';

import 'package:flutter/foundation.dart';
import 'package:flutter/widgets.dart';

import 'package:matches/matches.dart';
export 'package:matches/matches.dart';

part 'compute_tree_node.dart';
part 'compute_tree_widget.dart';
part 'node_builder.dart';
part 'unit.dart';
part 'unit_entry.dart';
part 'unit_key.dart';
part 'units.dart';
part 'untis/invoke.dart';
part 'untis/invoke_async.dart';
part 'untis/memo.dart';
part 'untis/ref.dart';
part 'untis/trigger.dart';

T bindUnit<T, K extends Object>(Unit<T, K> Function() fn, K dep) {
  return CTNode.currentTarget._units.next<T, K>(fn, dep);
}

const dynamic _constDep = Object();

Ref<T> ref<T, K extends Object>(
  T Function() factory, [
  K? dep,
]) {
  dep ??= _constDep;
  return bindUnit<Ref<T>, K>(() => _RefUnit<T, K>(factory), dep as K);
}

T memo<T>(T value) {
  return bindUnit<T, Object>(() => _MemoUnit(value), const Object());
}

class Invoke {
  const Invoke._();

  T immediate<T, K extends Object>(T Function() fn, [K? dep]) {
    dep ??= _constDep;
    return bindUnit<T, K>(() => _InvokeUnit(fn), dep as K);
  }

  T deferred<T, K extends Object>(
    T Function() fn, {
    K? dep,
    required Future<void> until,
    required T initialValue,
  }) {
    dep ??= _constDep;

    return bindUnit<T, K>(
      () => _InvokeUnit(fn, until, initialValue),
      dep as K,
    );
  }
}

const invoke = Invoke._();

Result<T> invokeAsync<T, K extends Object, R>(
  Future<T> Function() fn, {
  void Function()? abort,
  Future<void>? deferredUntil,
  K? dep,
}) {
  dep ??= _constDep;
  return bindUnit<Result<T>, K>(
    () => _InvokeAsyncUnit(fn, abort, deferredUntil),
    dep as K,
  );
}

Trigger<T> trigger<T extends Object>() {
  return bindUnit<Trigger<T>, Object>(() => _TriggerUnit<T>(), const Object());
}
