part of './flutter_compute_tree.dart';

class Units {
  int _cursor = 0;
  final List<UnitEntry> _entries = [];
  final Map<UnitKey, Unit> _index = {};

  void enter() {
    _cursor = 0;
  }

  T next<T, K>(Unit<T, K> Function() fn, K dep) {
    Unit<T, K> unit;

    if (_cursor < _entries.length) {
      final entry = _entries[_cursor];

      if (entry is! UnitEntry<T, K>) {
        throw Exception(
          '${entry.key} is incompatible with '
          'UnitKey<$T, $K>($_cursor, $dep)',
        );
      }

      final newKey = UnitKey<T, K>(_cursor, dep);

      if (entry.key != newKey) {
        final newUnit = fn();
        entry.update(entry.key._value, newKey, newUnit);
        _index.remove(entry.key);
        _index[newKey] = entry.unit;
      }

      unit = entry.unit;
    } else {
      final key = UnitKey<T, K>(_cursor, dep);
      unit = fn();

      final entry = UnitEntry<T, K>(
        key,
        unit,
        CTNode.currentTarget,
      );

      entry.init(dep);
      _entries.add(entry);
      _index[entry.key] = unit;
    }

    _cursor++;
    return unit.unwrap();
  }

  void exit() {
    while (_cursor < _entries.length) {
      final entry = _entries.removeLast();
      entry.dispose();
    }
  }

  void dispose() {
    for (final entry in _entries) {
      entry.dispose();
    }

    _entries.clear();
  }
}
