part of '../flutter_compute_tree.dart';

class _InvokeUnit<T, K> extends Unit<T, K> {
  late T value;
  final T Function() _callback;

  Future<void>? _deferredUntil;
  T? _initialValue;

  _InvokeUnit(
    this._callback, [
    this._deferredUntil,
    this._initialValue,
  ]);

  void _extractValue() {
    if (_deferredUntil != null) {
      value = _initialValue as T;
      _deferredUntil!.then((_) {
        notify();
      });
    } else {
      value = _callback();
    }
  }

  @override
  void init(CTNode node, K dep) {
    super.init(node, dep);
    _extractValue();
  }

  @override
  Unit<T, K> update(
    CTNode node,
    K oldDep,
    K newDep,
    _InvokeUnit<T, K> newUnit,
  ) {
    super.update(node, oldDep, newDep, newUnit);

    if (_deferredUntil == newUnit._deferredUntil) {
      value = newUnit._callback();
      return this;
    } else {
      _deferredUntil = newUnit._deferredUntil;
      _initialValue = newUnit._initialValue;
      _extractValue();

      return this;
    }
  }

  @override
  T unwrap() {
    return value;
  }
}
